/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes
#include "OLED128x64.h"
#include <stdio.h>
#include <string.h>

volatile bool bIsI2C_DONE = true ;
volatile bool bIsTC0_DONE = false ;

#define     MCP9800_SLAVE_ADDRESS   0x4d 
uint8_t      i2cWrData[8] ;
uint8_t     i2cRdData[8] ;
uint8_t     OLED_Buffer[20] ;
uint8_t     TC0_OverflowTime = 0 ;

uint8_t                 Can0MessageRAM[CAN0_MESSAGE_RAM_CONFIG_SIZE]  __attribute__((aligned (32))) ;
unsigned char       CAN_TxBuffer[64] ;
unsigned char       CAN_Counter = 0 ;

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

void    EIC11_Handler (uintptr_t context )
{
    LED1_PB10_Toggle();
}

void    TC0_EventHandler( TC_TIMER_STATUS Status, uintptr_t context)
{
    LED2_PA10_Toggle();
    bIsTC0_DONE = true ;
}

void I2C_EventHandler (uintptr_t context)
{
    if (SERCOM2_I2C_ErrorGet() == SERCOM_I2C_ERROR_NONE) 
        
        bIsI2C_DONE = true ;     
}

int main ( void )
{
    /* Initialize all modules */
    SYS_Initialize ( NULL );
    
    EIC_CallbackRegister(EIC_PIN_11, EIC11_Handler , 0 );
    TC0_TimerCallbackRegister(TC0_EventHandler,  (uintptr_t ) NULL ) ;
    SERCOM2_I2C_CallbackRegister( I2C_EventHandler, 0 ) ;
    TC0_TimerStart() ; 
    CAN0_MessageRAMConfigSet(Can0MessageRAM ) ;
    
        bIsTC0_DONE = false ;
        while (bIsTC0_DONE == false ) { }
    
        OLED_Init();
        OLED_CLS();
        OLED_Put8x16Str(0,0, (const unsigned char*) "APP-Nano-C21-D21");
        OLED_Put8x16Str(0,2, (const unsigned char*) "I am Calvin Ho");
        OLED_Put8x16Str(0,6, (const unsigned char*) "Exer-5");
    
    while ( true )
    {
        /* Maintain state machines of all polled MPLAB Harmony modules. */
        SYS_Tasks ( );
        if (bIsTC0_DONE)
        {
            bIsTC0_DONE = false ;
            if (++ TC0_OverflowTime ==2)
            {
                TC0_OverflowTime = 0 ;
                bIsI2C_DONE = false ;
                i2cWrData[0] = 0x00 ;
                SERCOM2_I2C_WriteRead( MCP9800_SLAVE_ADDRESS, i2cWrData, 1, i2cRdData, 2) ;
                while ( bIsI2C_DONE == false) {};
                sprintf ((char*)OLED_Buffer, "MCP9800= %d Deg.%c", i2cRdData[0], '\0' ) ; 
                OLED_Put8x16Str(0,4 , OLED_Buffer) ;
                
                if (CAN0_TxFIFOIsFull() == false)
                {
                    CAN_TxBuffer[0] = i2cRdData[0] ;
                    CAN_TxBuffer[1] = 88 ;
                    CAN_TxBuffer[2] = 99 ;
                    CAN_TxBuffer[3] = CAN_Counter ++ ;
                    CAN0_MessageTransmit(0x100, 4, CAN_TxBuffer, CAN_MODE_NORMAL, CAN_MSG_ATTR_TX_FIFO_DATA_FRAME) ;
                }
                
            }
        }

    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

